#    Oak - a single-system message-board
#    Copyright (C) 2002  Michael Urman
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
dumb is a dumb-terminal interface to the oak message-board system.
"""

import time
try:
    import readline
except ImportError:
    pass

class UserInterface:
    """UserInterface gets commands from and shows output to the user"""

    def __init__(self, tree):
        self._tree = tree

    def run(self):
        commands = {
            'create': self.create,
            'date': self.date,
            'branches': self.branches,
            'branch': self.branch,
            'leaves': self.leaves,
            'leaf': self.leaf,
            'postbranch': self.postbranch,
        }

        while 1:
            try:
                input = raw_input('oak> ')
            except EOFError:
                break

            tokens = input.split()
            if commands.has_key(tokens[0]):
                #try:
                    commands[tokens[0]](*tokens[1:])
                #except Exception, e:
                    #print 'Error:', e
            else:
                print 'command %s not defined' % tokens[0]

    def create(self, *args):
        if len(args)==1:
            self._tree.new_trunk(str(args[0]))
        else:
            print 'usage: create {trunk}'

    def date(self, *args):
        if 1 <= len(args) <= 2:
            tname = args[0]
            if len(args) == 2:
                type = args[1]
            else:
                type = 'modified'
            
            trunk = self._tree.trunk(tname)
            date = trunk.date(type)
            print '[%s] date %s: %s (%s)' % (tname, type, time.ctime(date), date)
        else:
            print 'usage: date [modified|created|reset]'

    def branches(self, *args):
        if len(args) == 1:
            branches = self._tree.trunk(args[0]).branch_count()
            print '[%s] %s branches' % (args[0], branches)
        else:
            print 'usage: branches {trunk}'

    def leaves(self, *args):
        if len(args) == 2:
            leaves = self._tree.trunk(args[0]).branch(args[1]).leaf_count()
            print '[%s] branch %s has %s leaves' % (args[0], args[1], leaves)
        else:
            print 'usage: leaves {trunk} {branch}'

    def postbranch(self, *args):
        if len(args) == 3:
            trunk, title, message = args
            branch = self._tree.trunk(trunk).branch()
            branch.new(title, fromstring=message)
            print '[%s] added branch %s (%s) with message %s' % (trunk, branch.title(), branch._num, message)
        else:
            print 'usage: postbranch {trunk} {title} {message}'

    def branch(self, *args):
        if len(args) == 2:
            trunk, num = args
            branch = self._tree.trunk(trunk).branch(num)
            leaf = branch.leaf(0)
            print '[%s:%s] "%s" posted %s by %s' % (trunk, num, branch.title(), leaf.date(), leaf.author())
        else:
            print 'usage: branch {trunk} {branch}'

    def leaf(self, *args):
        if len(args) == 3:
            trunk, branch, num = args
            leaf = self._tree.trunk(trunk).branch(branch).leaf(num)
            leaf.read()
            print '[%s:%s:%s] posted %s by %s:' % (trunk, branch, num, time.ctime(leaf.date()), leaf.author())
            print leaf.message()
        else:
            print 'usage: leaf {trunk} {branch} {leaf}'
